feat(cccl): integrate missing CCCL directories — python/, ci/, .agent/, docs/, test/
Sparse-checkout from NVIDIA/cccl main branch to complete cccl_upstream: Added: - python/cuda_cccl/ (226 files) — Python bindings for device-level algorithms Critical for muh toolchain: cuda.compute.reduce_into, scan, radix_sort, etc. Includes 204 .py files with full test coverage for all 27 algorithms - ci/ (163 files) — Build/test infrastructure build_cub.sh, test_cub.sh, build_and_test_targets.sh, matrix.yaml Directly maps to our [INFRA-CI] and [INFRA-BUILD] items - .agent/skills/ (7 files) — NVIDIA's own agent skills for CCCL cccl-style/SKILL.md, cccl-test/SKILL.md, sass-diff/SKILL.md - docs/ (491 files) — Official CCCL documentation CI references, CMake guides, Python compute docs, libcudacxx PTX docs - test/ (12 files) — Top-level integration tests (cuda_smoke, stdpar) - Root configs: .clang-format, .clang-tidy, CONTRIBUTING.md, pyproject.toml - CLAUDE.md symlink → AGENTS.md (NVIDIA's standard) cccl_upstream now mirrors full NVIDIA/cccl structure: Before: 42M (cub + thrust + libcudacxx + cudax + c + examples + benchmarks) After: 53M (+python +ci +docs +.agent +test +configs) This completes the CCCL base needed for: - [muh-bench] items: ci/util/build_and_test_targets.sh for targeted builds - [CCCL-verify] items: python/cuda_cccl/tests/ as reference implementations - [CCCL-test] items: ci/test_cub.sh, ci/test_thrust.sh - Agent workflow: .agent/skills/ for consistent style and test patterns
This commit is contained in:
70
cccl_upstream/docs/libcudacxx/extended_api/math/pow2.rst
Normal file
70
cccl_upstream/docs/libcudacxx/extended_api/math/pow2.rst
Normal file
@@ -0,0 +1,70 @@
|
||||
.. _libcudacxx-extended-api-math-pow2:
|
||||
|
||||
Power of Two Utilities
|
||||
======================
|
||||
|
||||
Defined in the ``<cuda/cmath>`` header.
|
||||
|
||||
.. code:: cuda
|
||||
|
||||
namespace cuda {
|
||||
|
||||
template <typename T>
|
||||
[[nodiscard]] __host__ __device__ constexpr
|
||||
bool is_power_of_two(T value) noexcept;
|
||||
|
||||
template <typename T>
|
||||
[[nodiscard]] __host__ __device__ constexpr
|
||||
T next_power_of_two(T value) noexcept;
|
||||
|
||||
template <typename T>
|
||||
[[nodiscard]] __host__ __device__ constexpr
|
||||
T prev_power_of_two(T value) noexcept;
|
||||
|
||||
} // namespace cuda
|
||||
|
||||
The functions provide utilities to determine if an integer value is a power of two, and to compute the next and previous power of two.
|
||||
|
||||
**Parameters**
|
||||
|
||||
- ``value``: The input value.
|
||||
|
||||
**Return value**
|
||||
|
||||
- ``is_power_of_two``: Return ``true`` if ``value`` is a power of two, ``false`` otherwise.
|
||||
- ``next_power_of_two``: Return the smallest power of two greater than or equal to ``value``.
|
||||
- ``prev_power_of_two``: Return the largest power of two less than or equal to ``value``.
|
||||
|
||||
**Constraints**
|
||||
|
||||
- ``T`` is an integer types. Contrary to ``cuda::std::has_single_bit``, ``cuda::std::bit_floor``, and ``cuda::std::bit_ceil``, ``T`` can be both signed and unsigned.
|
||||
|
||||
**Preconditions**
|
||||
|
||||
- ``value > 0``
|
||||
|
||||
**Performance considerations**
|
||||
|
||||
See :ref:`\<cuda/std/bit\> performance considerations <libcudacxx-standard-api-numerics-bit>`
|
||||
|
||||
Example
|
||||
-------
|
||||
|
||||
.. code:: cuda
|
||||
|
||||
#include <cuda/cmath>
|
||||
#include <cuda/std/cassert>
|
||||
|
||||
__global__ void pow2_kernel() {
|
||||
assert(!cuda::is_power_of_two(20));
|
||||
assert(cuda::next_power_of_two(20) == 32);
|
||||
assert(cuda::prev_power_of_two(20) == 16);
|
||||
}
|
||||
|
||||
int main() {
|
||||
pow2_kernel<<<1, 1>>>();
|
||||
cudaDeviceSynchronize();
|
||||
return 0;
|
||||
}
|
||||
|
||||
`See it on Godbolt 🔗 <https://godbolt.org/z/896Yx3vf8>`__
|
||||
Reference in New Issue
Block a user